from django.http import HttpResponse
from shiftregister.app.models import Helper, ShiftRegistration, Message
from django.db.models import Count, Case, When, Sum
from django.db import models


def metrics(request):
    response = HttpResponse(
        "\n".join(
            (
                f"shiftregister_{name} {value}"
                for name, value in (
                    ("helpers_total", Helper.objects.count()),
                    (
                        "helpers_confirmed_total",
                        Helper.objects.filter(number_validated=True).count(),
                    ),
                    (
                        "helpers_helped_total",
                        Helper.objects.annotate(
                            shift_count=Count(
                                Case(
                                    When(
                                        shiftregistration__state__in=[
                                            ShiftRegistration.RegState.CHECKED_IN
                                        ],
                                        then=1,
                                    ),
                                    output_field=models.IntegerField(),
                                )
                            )
                        )
                        .filter(number_validated=True, shift_count__gte=1)
                        .count(),
                    ),
                    (
                        "messages_sent_total",
                        Message.objects.all().count(),
                    ),
                    (
                        "worked_shifts_total",
                        ShiftRegistration.objects.filter(
                            state=ShiftRegistration.RegState.CHECKED_IN
                        ).count(),
                    ),
                    (
                        "worked_seconds_total",
                        ShiftRegistration.objects.filter(
                            state=ShiftRegistration.RegState.CHECKED_IN
                        )
                        .aggregate(sum=Sum("shift__duration"))["sum"]
                        .total_seconds(),
                    ),
                )
            )
        )
    )
    response.headers["Content-Type"] = "text/plain"

    return response
